@@ -69,6 +69,11 @@ SquiggleScalings estimate_scalings_using_mom(const std::string& sequence,
 #define move_down(curr_band) { curr_band.event_idx + 1, curr_band.kmer_idx }
 #define move_right(curr_band) { curr_band.event_idx, curr_band.kmer_idx + 1 }
 
+#define ALN_BANDWIDTH 100
+
+#define BAND_ARRAY(r, c) ( bands[((r)*(ALN_BANDWIDTH)+(c))] )
+#define TRACE_ARRAY(r, c) ( trace[((r)*(ALN_BANDWIDTH)+(c))] )
+
 std::vector<AlignedPair> adaptive_banded_simple_event_align(SquiggleRead& read, const PoreModel& pore_model, const std::string& sequence)
 {
     size_t strand_idx = 0;
@@ -87,7 +92,7 @@ std::vector<AlignedPair> adaptive_banded_simple_event_align(SquiggleRead& read,
     int max_gap_threshold = 50;
 
     // banding
-    int bandwidth = 100;
+    int bandwidth = ALN_BANDWIDTH;
     int half_bandwidth = bandwidth / 2;
  
     // transition penalties
@@ -115,15 +120,21 @@ std::vector<AlignedPair> adaptive_banded_simple_event_align(SquiggleRead& read,
         kmer_ranks[i] = alphabet->kmer_rank(sequence.substr(i, k).c_str(), k);
     }
 
-    typedef std::vector<float> bandscore;
-    typedef std::vector<uint8_t> bandtrace;
-
-    std::vector<bandscore> bands(n_bands);
-    std::vector<bandtrace> trace(n_bands);
-
-    for(size_t i = 0; i < n_bands; ++i) {
-        bands[i].resize(bandwidth, -INFINITY);
-        trace[i].resize(bandwidth, 0);
+    float* bands = (float*)malloc(sizeof(float) * n_bands * bandwidth);
+    if(bands==NULL){
+        fprintf(stderr,"Memory allocation failed at %s\n",__func__);
+        exit(1);
+    }
+    uint8_t* trace = (uint8_t*)malloc(sizeof(uint8_t) * n_bands * bandwidth);
+    if(trace==NULL){
+        fprintf(stderr,"Memory allocation failed at %s\n",__func__);
+        exit(1);
+    }
+    for (size_t i = 0; i < n_bands; i++) {
+        for (int j = 0; j < bandwidth; j++) {
+            BAND_ARRAY(i,j) = -INFINITY;
+            TRACE_ARRAY(i,j) = 0;
+        }
     }
 
     // Keep track of the event/kmer index for the lower left corner of the band
@@ -146,18 +157,18 @@ std::vector<AlignedPair> adaptive_banded_simple_event_align(SquiggleRead& read,
     int start_cell_offset = band_kmer_to_offset(0, -1);
     assert(is_offset_valid(start_cell_offset));
     assert(band_event_to_offset(0, -1) == start_cell_offset);
-    bands[0][start_cell_offset] = 0.0f;
-    
+    BAND_ARRAY(0,start_cell_offset) = 0.0f;
+
     // band 1: first event is trimmed
     int first_trim_offset = band_event_to_offset(1, 0);
     assert(kmer_at_offset(1, first_trim_offset) == -1);
     assert(is_offset_valid(first_trim_offset));
-    bands[1][first_trim_offset] = lp_trim;
-    trace[1][first_trim_offset] = FROM_U;
+    BAND_ARRAY(1,first_trim_offset) = lp_trim;
+    TRACE_ARRAY(1,first_trim_offset) = FROM_U;
 
     int fills = 0;
 #ifdef DEBUG_ADAPTIVE
-    fprintf(stderr, "[trim] bi: %d o: %d e: %d k: %d s: %.2lf\n", 1, first_trim_offset, 0, -1, bands[1][first_trim_offset]);
+    fprintf(stderr, "[trim] bi: %d o: %d e: %d k: %d s: %.2lf\n", 1, first_trim_offset, 0, -1, BAND_ARRAY(1,first_trim_offset));
 #endif
 
     // fill in remaining bands
@@ -165,11 +176,11 @@ std::vector<AlignedPair> adaptive_banded_simple_event_align(SquiggleRead& read,
         // Determine placement of this band according to Suzuki's adaptive algorithm
         // When both ll and ur are out-of-band (ob) we alternate movements
         // otherwise we decide based on scores
-        float ll = bands[band_idx - 1][0];
-        float ur = bands[band_idx - 1][bandwidth - 1];
+        float ll = BAND_ARRAY(band_idx - 1,0);
+        float ur = BAND_ARRAY(band_idx - 1,bandwidth - 1);
         bool ll_ob = ll == -INFINITY;
         bool ur_ob = ur == -INFINITY;
-        
+
         bool right = false;
         if(ll_ob && ur_ob) {
             right = band_idx % 2 == 1;
@@ -206,10 +217,10 @@ std::vector<AlignedPair> adaptive_banded_simple_event_align(SquiggleRead& read,
         if(is_offset_valid(trim_offset)) {
             int event_idx = event_at_offset(band_idx, trim_offset);
             if(event_idx >= 0 && event_idx < n_events) {
-                bands[band_idx][trim_offset] = lp_trim * (event_idx + 1);
-                trace[band_idx][trim_offset] = FROM_U;
+                BAND_ARRAY(band_idx,trim_offset) = lp_trim * (event_idx + 1);
+                TRACE_ARRAY(band_idx,trim_offset) = FROM_U;
             } else {
-                bands[band_idx][trim_offset] = -INFINITY;
+                BAND_ARRAY(band_idx,trim_offset) = -INFINITY;
             }
         }
 
@@ -231,11 +242,11 @@ std::vector<AlignedPair> adaptive_banded_simple_event_align(SquiggleRead& read,
             int kmer_idx = kmer_at_offset(band_idx, offset);
 
             size_t kmer_rank = kmer_ranks[kmer_idx];
- 
-            int offset_up   = band_event_to_offset(band_idx - 1, event_idx - 1); 
+
+            int offset_up   = band_event_to_offset(band_idx - 1, event_idx - 1);
             int offset_left = band_kmer_to_offset(band_idx - 1, kmer_idx - 1);
             int offset_diag = band_kmer_to_offset(band_idx - 2, kmer_idx - 1);
- 
+
 #ifdef DEBUG_ADAPTIVE
             // verify loop conditions
             assert(kmer_idx >= 0 && kmer_idx < n_kmers);
@@ -245,10 +256,10 @@ std::vector<AlignedPair> adaptive_banded_simple_event_align(SquiggleRead& read,
             assert(offset >= 0 && offset < bandwidth);
 #endif
 
-            float up   = is_offset_valid(offset_up)   ? bands[band_idx - 1][offset_up]   : -INFINITY;
-            float left = is_offset_valid(offset_left) ? bands[band_idx - 1][offset_left] : -INFINITY;
-            float diag = is_offset_valid(offset_diag) ? bands[band_idx - 2][offset_diag] : -INFINITY;
- 
+            float up   = is_offset_valid(offset_up)   ? BAND_ARRAY(band_idx - 1,offset_up)   : -INFINITY;
+            float left = is_offset_valid(offset_left) ? BAND_ARRAY(band_idx - 1,offset_left) : -INFINITY;
+            float diag = is_offset_valid(offset_diag) ? BAND_ARRAY(band_idx - 2,offset_diag) : -INFINITY;
+
             float lp_emission = log_probability_match_r9(read, pore_model, kmer_rank, event_idx, strand_idx);
             float score_d = diag + lp_step + lp_emission;
             float score_u = up + lp_stay + lp_emission;
@@ -261,18 +272,18 @@ std::vector<AlignedPair> adaptive_banded_simple_event_align(SquiggleRead& read,
             from = max_score == score_u ? FROM_U : from;
             max_score = score_l > max_score ? score_l : max_score;
             from = max_score == score_l ? FROM_L : from;
-    
+
 #ifdef DEBUG_ADAPTIVE
             fprintf(stderr, "[adafill] offset-up: %d offset-diag: %d offset-left: %d\n", offset_up, offset_diag, offset_left);
             fprintf(stderr, "[adafill] up: %.2lf diag: %.2lf left: %.2lf\n", up, diag, left);
             fprintf(stderr, "[adafill] bi: %d o: %d e: %d k: %d s: %.2lf f: %d emit: %.2lf\n", band_idx, offset, event_idx, kmer_idx, max_score, from, lp_emission);
 #endif
-            bands[band_idx][offset] = max_score;
-            trace[band_idx][offset] = from;
+            BAND_ARRAY(band_idx,offset) = max_score;
+            TRACE_ARRAY(band_idx,offset) = from;
             fills += 1;
         }
     }
-    
+
     /*
     // Debug, print some of the score matrix
     for(int col = 0; col <= 10; ++col) {
@@ -294,7 +305,7 @@ std::vector<AlignedPair> adaptive_banded_simple_event_align(SquiggleRead& read,
     double sum_emission = 0;
     double n_aligned_events = 0;
     std::vector<AlignedPair> out;
-    
+
     float max_score = -INFINITY;
     int curr_event_idx = 0;
     int curr_kmer_idx = n_kmers -1;
@@ -302,17 +313,16 @@ std::vector<AlignedPair> adaptive_banded_simple_event_align(SquiggleRead& read,
     // Find best score between an event and the last k-mer. after trimming the remaining evnets
     for(int event_idx = 0; event_idx < n_events; ++event_idx) {
         int band_idx = event_kmer_to_band(event_idx, curr_kmer_idx);
-        assert(band_idx < bands.size());
         int offset = band_event_to_offset(band_idx, event_idx);
         if(is_offset_valid(offset)) {
-            float s = bands[band_idx][offset] + (n_events - event_idx) * lp_trim;
+            float s = BAND_ARRAY(band_idx,offset)  + (n_events - event_idx) * lp_trim;
             if(s > max_score) {
                 max_score = s;
                 curr_event_idx = event_idx;
             }
         }
     }
-    
+
 #ifdef DEBUG_ADAPTIVE
     fprintf(stderr, "[adaback] ei: %d ki: %d s: %.2f\n", curr_event_idx, curr_kmer_idx, max_score);
 #endif
@@ -320,12 +330,12 @@ std::vector<AlignedPair> adaptive_banded_simple_event_align(SquiggleRead& read,
     int curr_gap = 0;
     int max_gap = 0;
     while(curr_kmer_idx >= 0 && curr_event_idx >= 0) {
-        
+
         // emit alignment
         out.push_back({curr_kmer_idx, curr_event_idx});
 #ifdef DEBUG_ADAPTIVE
         fprintf(stderr, "[adaback] ei: %d ki: %d\n", curr_event_idx, curr_kmer_idx);
-#endif       
+#endif
         // qc stats
         size_t kmer_rank = alphabet->kmer_rank(sequence.substr(curr_kmer_idx, k).c_str(), k);
         sum_emission += log_probability_match_r9(read, pore_model, kmer_rank, curr_event_idx, strand_idx);
@@ -335,7 +345,7 @@ std::vector<AlignedPair> adaptive_banded_simple_event_align(SquiggleRead& read,
         int offset = band_event_to_offset(band_idx, curr_event_idx);
         assert(band_kmer_to_offset(band_idx, curr_kmer_idx) == offset);
 
-        uint8_t from = trace[band_idx][offset];
+        uint8_t from = TRACE_ARRAY(band_idx,offset);
         if(from == FROM_D) {
             curr_kmer_idx -= 1;
             curr_event_idx -= 1;
@@ -347,7 +357,7 @@ std::vector<AlignedPair> adaptive_banded_simple_event_align(SquiggleRead& read,
             curr_kmer_idx -= 1;
             curr_gap += 1;
             max_gap = std::max(curr_gap, max_gap);
-        }   
+        }
     }
     std::reverse(out.begin(), out.end());
     
@@ -361,6 +371,9 @@ std::vector<AlignedPair> adaptive_banded_simple_event_align(SquiggleRead& read,
         out.clear();
     }
 
+    free(bands);
+    free(trace);
+
     //fprintf(stderr, "ada\t%s\t%s\t%.2lf\t%zu\t%.2lf\t%d\t%d\t%d\n", read.read_name.substr(0, 6).c_str(), failed ? "FAILED" : "OK", events_per_kmer, sequence.size(), avg_log_emission, curr_event_idx, max_gap, fills);
     return out;
 }
